import os
from appsignal.__about__ import __version__

from appsignal.config import Config, Options


def test_option():
    config = Config(Options(active=False, enable_host_metrics=True))

    assert config.option("active") is False
    assert config.option("enable_host_metrics") is True
    assert config.option("nonsense") is None


def test_system_source():
    config = Config()

    assert list(config.sources["system"].keys()) == ["app_path"]
    assert "app_path" in list(config.options.keys())


def test_environ_source():
    os.environ["APPSIGNAL_ACTIVE"] = "true"
    os.environ["APPSIGNAL_APP_ENV"] = "development"
    os.environ["APPSIGNAL_APP_NAME"] = "MyApp"
    os.environ["APPSIGNAL_CA_FILE_PATH"] = "/path/to/cacert.pem"
    os.environ["APPSIGNAL_DNS_SERVERS"] = "8.8.8.8,8.8.4.4"
    os.environ["APPSIGNAL_ENABLE_HOST_METRICS"] = "true"
    os.environ["APPSIGNAL_ENABLE_NGINX_METRICS"] = "false"
    os.environ["APPSIGNAL_ENABLE_STATSD"] = "false"
    os.environ["APPSIGNAL_FILES_WORLD_ACCESSIBLE"] = "true"
    os.environ["APPSIGNAL_FILTER_PARAMETERS"] = "password,secret"
    os.environ["APPSIGNAL_FILTER_SESSION_DATA"] = "key1,key2"
    os.environ["APPSIGNAL_HOSTNAME"] = "Test hostname"
    os.environ["APPSIGNAL_HTTP_PROXY"] = "http://proxy.local:9999"
    os.environ["APPSIGNAL_IGNORE_ACTIONS"] = "action1,action2"
    os.environ["APPSIGNAL_IGNORE_ERRORS"] = "error1,error2"
    os.environ["APPSIGNAL_IGNORE_NAMESPACES"] = "namespace1,namespace2"
    os.environ["APPSIGNAL_LOG_LEVEL"] = "trace"
    os.environ["APPSIGNAL_LOG_PATH"] = "/path/to/log_dir"
    os.environ["APPSIGNAL_PUSH_API_KEY"] = "some-api-key"
    os.environ["APPSIGNAL_PUSH_API_ENDPOINT"] = "https://push.appsignal.com"
    os.environ["APPSIGNAL_REQUEST_HEADERS"] = "accept,x-custom-header"
    os.environ["APPSIGNAL_RUNNING_IN_CONTAINER"] = "true"
    os.environ["APPSIGNAL_SEND_ENVIRONMENT_METADATA"] = "true"
    os.environ["APPSIGNAL_SEND_PARAMS"] = "true"
    os.environ["APPSIGNAL_SEND_SESSION_DATA"] = "true"
    os.environ["APPSIGNAL_WORKING_DIRECTORY_PATH"] = "/path/to/working/dir"
    os.environ["APP_REVISION"] = "abc123"

    config = Config()

    env_options = Options(
        active=True,
        ca_file_path="/path/to/cacert.pem",
        dns_servers=["8.8.8.8", "8.8.4.4"],
        enable_host_metrics=True,
        enable_nginx_metrics=False,
        enable_statsd=False,
        endpoint="https://push.appsignal.com",
        environment="development",
        files_world_accessible=True,
        filter_parameters=["password", "secret"],
        filter_session_data=["key1", "key2"],
        hostname="Test hostname",
        http_proxy="http://proxy.local:9999",
        ignore_actions=["action1", "action2"],
        ignore_errors=["error1", "error2"],
        ignore_namespaces=["namespace1", "namespace2"],
        log_level="trace",
        log_path="/path/to/log_dir",
        name="MyApp",
        push_api_key="some-api-key",
        revision="abc123",
        request_headers=["accept", "x-custom-header"],
        running_in_container=True,
        send_environment_metadata=True,
        send_params=True,
        send_session_data=True,
        working_directory_path="/path/to/working/dir",
    )
    assert config.sources["environment"] == env_options
    final_options = Options()
    final_options.update(config.sources["default"])
    final_options.update(config.sources["system"])
    final_options.update(env_options)
    assert config.options == final_options


def test_environ_source_bool_is_unset():
    config = Config()

    assert config.sources["environment"].get("active") is None
    assert config.option("active") is None


def test_environ_source_bool_is_empty_string():
    os.environ["APPSIGNAL_ACTIVE"] = ""

    config = Config()

    assert config.sources["environment"].get("active") is None
    assert config.option("active") is None


def test_environ_source_bool_is_invalid():
    os.environ["APPSIGNAL_ACTIVE"] = "invalid"

    config = Config()

    assert config.sources["environment"].get("active") is None
    assert config.option("active") is None


def test_environ_source_disable_default_instrumentations_list():
    os.environ["APPSIGNAL_DISABLE_DEFAULT_INSTRUMENTATIONS"] = ",".join(
        ["opentelemetry.instrumentation.celery", "something.else"]
    )

    config = Config()

    assert config.sources["environment"]["disable_default_instrumentations"] == [
        "opentelemetry.instrumentation.celery"
    ]
    assert config.options["disable_default_instrumentations"] == [
        "opentelemetry.instrumentation.celery"
    ]


def test_environ_source_disable_default_instrumentations_bool():
    for value, expected in [
        ("True", True),
        ("true", True),
        ("False", False),
        ("false", False),
    ]:
        os.environ["APPSIGNAL_DISABLE_DEFAULT_INSTRUMENTATIONS"] = value
        config = Config()
        assert config.options["disable_default_instrumentations"] is expected


def test_set_private_environ():
    cwdir = os.getcwd()
    config = Config(
        Options(
            active=True,
            app_path="/path/to/app",
            ca_file_path="/path/to/cacert.pem",
            dns_servers=["8.8.8.8", "8.8.4.4"],
            enable_host_metrics=True,
            enable_nginx_metrics=False,
            enable_statsd=False,
            endpoint="https://push.appsignal.com",
            environment="development",
            files_world_accessible=True,
            filter_parameters=["password", "secret"],
            filter_session_data=["key1", "key2"],
            hostname="Test hostname",
            http_proxy="http://proxy.local:9999",
            ignore_actions=["action1", "action2"],
            ignore_errors=["error1", "error2"],
            ignore_namespaces=["namespace1", "namespace2"],
            log_level="trace",
            log_path=cwdir,
            name="MyApp",
            push_api_key="some-api-key",
            revision="abc123",
            running_in_container=True,
            send_environment_metadata=True,
            send_params=True,
            send_session_data=True,
            working_directory_path="/path/to/working/dir",
        )
    )

    config.set_private_environ()

    assert os.environ["_APPSIGNAL_ACTIVE"] == "true"
    assert os.environ["_APPSIGNAL_APP_ENV"] == "development"
    assert os.environ["_APPSIGNAL_APP_NAME"] == "MyApp"
    assert os.environ["_APPSIGNAL_APP_PATH"] == "/path/to/app"
    assert os.environ["_APPSIGNAL_CA_FILE_PATH"] == "/path/to/cacert.pem"
    assert os.environ["_APPSIGNAL_DNS_SERVERS"] == "8.8.8.8,8.8.4.4"
    assert os.environ["_APPSIGNAL_ENABLE_HOST_METRICS"] == "true"
    assert os.environ["_APPSIGNAL_ENABLE_NGINX_METRICS"] == "false"
    assert os.environ["_APPSIGNAL_ENABLE_STATSD"] == "false"
    assert os.environ["_APPSIGNAL_FILES_WORLD_ACCESSIBLE"] == "true"
    assert os.environ["_APPSIGNAL_FILTER_PARAMETERS"] == "password,secret"
    assert os.environ["_APPSIGNAL_FILTER_SESSION_DATA"] == "key1,key2"
    assert os.environ["_APPSIGNAL_HOSTNAME"] == "Test hostname"
    assert os.environ["_APPSIGNAL_HTTP_PROXY"] == "http://proxy.local:9999"
    assert os.environ["_APPSIGNAL_IGNORE_ACTIONS"] == "action1,action2"
    assert os.environ["_APPSIGNAL_IGNORE_ERRORS"] == "error1,error2"
    assert os.environ["_APPSIGNAL_IGNORE_NAMESPACES"] == "namespace1,namespace2"
    assert os.environ["_APPSIGNAL_LOG_LEVEL"] == "trace"
    assert os.environ["_APPSIGNAL_LOG_FILE_PATH"] == f"{cwdir}/appsignal.log"
    assert os.environ["_APPSIGNAL_PUSH_API_KEY"] == "some-api-key"
    assert os.environ["_APPSIGNAL_PUSH_API_ENDPOINT"] == "https://push.appsignal.com"
    assert (
        os.environ["_APPSIGNAL_LANGUAGE_INTEGRATION_VERSION"] == f"python-{__version__}"
    )
    assert os.environ["_APPSIGNAL_RUNNING_IN_CONTAINER"] == "true"
    assert os.environ["_APPSIGNAL_SEND_ENVIRONMENT_METADATA"] == "true"
    assert os.environ["_APPSIGNAL_SEND_PARAMS"] == "true"
    assert os.environ["_APPSIGNAL_SEND_SESSION_DATA"] == "true"
    assert os.environ["_APPSIGNAL_WORKING_DIRECTORY_PATH"] == "/path/to/working/dir"
    assert os.environ["_APP_REVISION"] == "abc123"


def test_set_private_environ_valid_log_path():
    cwdir = os.getcwd()
    config = Config(Options(log_path=cwdir))
    config.set_private_environ()

    assert os.environ["_APPSIGNAL_LOG_FILE_PATH"] == f"{cwdir}/appsignal.log"


def test_set_private_environ_remove_filename_from_log_path():
    cwdir = os.getcwd()
    log_path = os.path.join(cwdir, "test.log")
    config = Config(Options(log_path=log_path))
    config.set_private_environ()

    assert os.environ["_APPSIGNAL_LOG_FILE_PATH"] == f"{cwdir}/appsignal.log"


def test_set_private_environ_invalid_log_path():
    config = Config(Options(log_path="/i_dont_exist"))
    config.set_private_environ()

    assert os.environ["_APPSIGNAL_LOG_FILE_PATH"] == "/tmp/appsignal.log"


def test_set_private_environ_bool_is_none():
    config = Config(Options(active=None))

    config.set_private_environ()

    assert os.environ.get("_APPSIGNAL_ACTIVE") is None


def test_set_private_environ_list_is_none():
    config = Config(Options(dns_servers=None))

    config.set_private_environ()

    assert os.environ.get("_APPSIGNAL_DNS_SERVERS") is None
